/// https://pub.dev/packages/path/example
/// https://pub.dev/packages/json2yaml
/// https://pub.dev/packages/yaml
import 'dart:convert';
import 'dart:io';

import 'package:json2yaml/json2yaml.dart';
import 'package:r_assets/generate_r_file/generate_support_files.dart';
import 'package:yaml/yaml.dart';

import 'lib/convert_mate_code.dart';
import 'lib/template_string.dart';
import '../logger/log.dart';

var genCodeHeader = """
// ignore: unused_import
import 'package:flutter/widgets.dart';
// ignore: unused_import
import 'package:flutter/services.dart' show rootBundle;
// ignore: unused_import
import 'package:path/path.dart' as path;
// ignore: unused_import
import 'package:flutter_svg/flutter_svg.dart';
import 'svg_assets_support.dart';

part 'r.text.dart';  // 生成大段text文件原始符串
part 'r.string.dart';  // 生成String类
part 'r.icon.dart';  // 生成Icon类

/// This `R` class is generated and contains references to static asset resources.
class R {
  /// 封闭类型，禁止继承
  R._();
  /// package name: {#packageName#}
  // static const package = "{#packageName#}";
  static const package = null; // 暂时生成为null，抄的代码，原来可能支持插件包操作，影响assets字符串对应问题。

  /// This `R.image` struct is generated, and contains static references to static non-svg type image asset resources.
  static const image = _R_Image();

  /// This `R.svg` struct is generated, and contains static references to static svg type image asset resources.
  static const svg = _R_Svg();

  /// This `R.text` struct is generated, and contains static references to static text asset resources.
  static const text = _R_Text();

  /// This `R.fontFamily` struct is generated, and contains static references to static font asset resources.
  static const fontFamily = _R_FontFamily();
  
  /// 转换yaml，json类似文件到直接字符串输出
  static const string = _R_String();
  
  /// 转换icons 文件内的svg 到 widget
  static const icon = _R_Icon();
}
""";

var genAssetResource = """""";

//<editor-fold desc="R Image Class 模版">
var genRImageAssetResourceClass = """

// ignore: camel_case_types
class _R_Image_AssetResource {
  const _R_Image_AssetResource();
  
  {#insertImageAssetResourceR#}
}
""";

var genRImageClass = """
/// This `_R_Image` class is generated and contains references to static non-svg type image asset resources.
// ignore: camel_case_types
class _R_Image {
  const _R_Image();

  final asset = const _R_Image_AssetResource();
  
  {#insertImageR#}
}
""";
//</editor-fold>

//<editor-fold desc="R Svg Class 模版">
var genRSvgAssetResourceClass = """
// ignore: camel_case_types
class _R_Svg_AssetResource {
  const _R_Svg_AssetResource();
  
  {#insertSvgAssetResourceR#}
}
""";

var genRSvgClass = """
/// This `_R_Svg` class is generated and contains references to static svg type image asset resources.
// ignore: camel_case_types
class _R_Svg {
  const _R_Svg();

  final asset = const _R_Svg_AssetResource();
  
  {#insertSvgR#}
}
""";
//</editor-fold>

//<editor-fold desc="R String Class 模版">
var genRStringClass = """
part of 'r.assets.dart';

/// only support one level map like string
// ignore: camel_case_types
class _R_String {
  const _R_String();
  
  {#insertStringR#}
}
""";
var genRawTextClass = """
part of 'r.assets.dart';

/// only support one level map like string
// ignore: camel_case_types
class _Raw_Text {
  const _Raw_Text();
  
  {#insertRawTextR#}
}
""";
var genRTextAssetResourceClass = """
// ignore: camel_case_types
class _R_Text_AssetResource {
  const _R_Text_AssetResource();
  
  {#insertTextAssetResourceR#}
}
""";

var genRTextClass = """
// ignore: camel_case_types
class _R_Text {
  const _R_Text();

  final asset = const _R_Text_AssetResource();
  
  {#insertTextR#}
}
""";
//</editor-fold>

//<editor-fold desc="R Font Class 模版">

var genRFontClass = """
/// This `_R_FontFamily` class is generated and contains references to static font asset resources.
// ignore: camel_case_types
class _R_FontFamily {
  const _R_FontFamily();
  
  {#insertRFamilyName#}
}
""";
//</editor-fold>

//<editor-fold desc="R Icon Class 模版">
var genRIconClass = """
part of 'r.assets.dart';

/// only support one level map like string
// ignore: camel_case_types
class _R_Icon {
  const _R_Icon();
  
  {#insertIconR#}
}
""";

//</editor-fold>

///
/// 约定：
/// * 自动抓取项目的assets文件夹，不要修改pubspec.yaml中的assets、font部分
/// * 图片放入images、icons文件夹（可以不放，可兼容）
///   * svg也放入其中
///   * 针对icons路径下的svg额外处理，生成快速使用的Icon for Widget，R.icon.xxxxx;
/// * 文字类放入texts（可以不放，可兼容）
///   * 纯文本转换到静态变量  R.text.xxxxSync;
///   * 也转换了异步读取 R.text.xxxx() 返回Future string
///   * json yaml 类型，第一层级的key，会被转换为变量集到R.string
/// * 字体文件放入fonts,必须有二级路径，将转换为字体的变量名
///   * 例：./fonts/ping_fang/xxxx.ttf -> R.font.pingFang
///
///
///
/// TODO:
/// * 二进制类数据放入binaries
/// * 音频 audio
/// * 视频 video
void generateAssetsDartCode(String rootPath, [String reason = 'analyzer_start']) {
  // 获取上次配置文件更改，决定是否重新生成R文件 （主要是更改yaml会造成分析器反复重启）
  mirrorLog.info("generateAssetsDartCode rootPath at ${rootPath}");
  var yaml = File.fromUri(Uri.parse("${rootPath}/pubspec.yaml"));
  var pubspec = json.decode(json.encode(loadYaml(yaml.readAsStringSync())));
  mirrorLog.info("generateAssetsDartCode yaml.path ${yaml.path}");
  mirrorLog.info("last auto_generate_datetime ${pubspec['auto_generate_datetime']}");
  mirrorLog.info("last auto_generate_reason ${pubspec['auto_generate_reason']}");
  var dt = (DateTime.now().millisecondsSinceEpoch - (pubspec['auto_generate_datetime'] ?? 0));
  if (reason == 'analyzer_start' && pubspec['auto_generate_reason'] == 'analyzer_start' && dt < 3 * 60 * 1000) {
    mirrorLog.info("原因:${reason}; 跳过生成时间间隔太短(${dt}ms<3 * 60 * 1000ms)");
    return;
  }
  if (reason == 'analyzer_start' && pubspec['auto_generate_reason'] == 'add_or_remove' && dt < 1 * 60 * 1000) {
    mirrorLog.info("原因:${reason}; 跳过生成时间间隔太短(${dt}ms<3 * 60 * 1000ms)");
    return;
  }

  // 读取assets文件夹，生成预处理数据；
  var gAssetsFile = geneCodeAndPubSpecYaml(rootPath);
  // 拟合image
  String insertImageAssetResourceR = '';
  String insertImageR = '';
  List<String> insertYaml = <String>[];
  gAssetsFile.mateImages.forEach((element) {
    insertImageAssetResourceR += element.generatedCode4Asset;
    insertImageR += element.generatedCode4Image;
    insertYaml.add(element.assetFilePath);
  });
  // 拟合svg
  String insertSvgAssetResourceR = '';
  String insertSvgR = '';
  gAssetsFile.mateSvgs.forEach((element) {
    insertSvgAssetResourceR += element.generatedCode4Asset;
    insertSvgR += element.generatedCode4Svg;
    insertYaml.add(element.assetFilePath);
  });
  // 拟合 Icon
  String insertIconR = '';
  gAssetsFile.svgToIcons.forEach((element) {
    insertIconR += element.generatedCode4Icon;
  });

  // 拟合txt类型 | 拟合json,yaml文件类型数据
  String insertTextAssetResourceR = '';
  String insertTextR = '';
  String insertStringR = '';
  String insertRawTextR = '';
  gAssetsFile.mateStrings.forEach((element) {
    insertTextAssetResourceR += element.generatedCode4Asset;
    insertTextR += element.generatedCode4Text;
    insertYaml.add(element.assetFilePath);
    insertStringR += element.generatedCode4MapDataToString;
    insertRawTextR += element.generatedCode4RawText;
  });
  // 拟合字体文件Font
  String insertRFamilyName = '';
  gAssetsFile.mateFonts.map((e) => e.generatedCode4FamilyName).toSet().forEach((element) {
    // 使用map抽离数据，用set集合去重
    insertRFamilyName += element;
  });

  // ---------------基本类型---------------
  final templateClassR = TemplateString(genCodeHeader);
  // ---------------Image---------------
  final templateClassRImage = TemplateString(genRImageClass);
  final templateClassRImageAssetResource = TemplateString(genRImageAssetResourceClass);
  // ---------------Svg---------------
  final templateClassRSvg = TemplateString(genRSvgClass);
  final templateClassRSvgAssetResource = TemplateString(genRSvgAssetResourceClass);
  // ---------------Text | String---------------
  final templateClassRText = TemplateString(genRTextClass);
  final templateClassRTextAssetResource = TemplateString(genRTextAssetResourceClass);
  final templateClassRString = TemplateString(genRStringClass);
  final templateClassRawText = TemplateString(genRawTextClass);
  // --------------- Font ---------------
  final templateClassRFont = TemplateString(genRFontClass);
  // --------------- Icon ---------------
  final templateClassRIcon = TemplateString(genRIconClass);

  // 组合R文件
  var assetsType = <String, dynamic>{
    'packageName': 'dacall',
    'insertImageAssetResourceR': insertImageAssetResourceR,
    'insertImageR': insertImageR,
    'insertSvgAssetResourceR': insertSvgAssetResourceR,
    'insertSvgR': insertSvgR,
    'insertTextR': insertTextR,
    'insertTextAssetResourceR': insertTextAssetResourceR,
    'insertStringR': insertStringR,
    'insertRawTextR': insertRawTextR,
    'insertRFamilyName': insertRFamilyName,
    'insertIconR': insertIconR,
  };
  var resultCodeData = [
    templateClassR.format(assetsType),
    templateClassRImage.format(assetsType),
    templateClassRImageAssetResource.format(assetsType),
    templateClassRSvg.format(assetsType),
    templateClassRSvgAssetResource.format(assetsType),
    templateClassRText.format(assetsType),
    templateClassRTextAssetResource.format(assetsType),
    templateClassRFont.format(assetsType),
  ].join('\n');
  // 组合R文件 -- r.text.dart部分
  var resultCodeData4Text = [
    templateClassRawText.format(assetsType),
  ].join('\n');
  // 组合R文件 -- r.string.dart部分
  var resultCodeData4String = [
    templateClassRString.format(assetsType),
    ...gAssetsFile.mateStrings.map((e) => e.generatedCode4MapDataToHideClass).toList(),
  ].join('\n');
  // 组合R文件 -- r.icon.dart部分
  var resultCodeData4Icon = [
    templateClassRIcon.format(assetsType),
  ].join('\n');

  mirrorLog.info("generateAssetsDartCode rootPath => ${rootPath}/pubspec.yaml");

  // 检查svg库支持
  if (pubspec['dependencies'] != null && pubspec['dependencies']['flutter_svg'] == null) {
    print('为 dependencies 添加 svg 支持 flutter_svg: ^0.19.1');
    pubspec['dependencies']['flutter_svg'] = '^0.19.1';
  }
  // 塞入生成的assets数据
  if (pubspec['flutter'] != null) {
    if (insertYaml.isNotEmpty) {
      pubspec['flutter']['assets'] = insertYaml;
    } else {
      pubspec['flutter'].remove('assets');
    }

    if (gAssetsFile.mateFonts.isNotEmpty) {
      var fontList = [];
      gAssetsFile.mateFonts.forEach((element) {
        // 是否已经添加过family name
        var hasFamilyName = false;
        for (var value in fontList) {
          if (value['family'] == element.generatedVarName) {
            hasFamilyName = true;
            value['fonts'].add(<String, dynamic>{"asset": element.assetFilePath});
            break;
          }
        }
        if (hasFamilyName) {
        } else {
          fontList.add(<String, dynamic>{
            'family': element.generatedVarName,
            'fonts': [
              <String, dynamic>{"asset": element.assetFilePath},
            ],
          });
        }
      });
      pubspec['flutter']['fonts'] = fontList;
    } else {
      pubspec['flutter'].remove('fonts');
    }
  }

  // 迁移代码生成代码
  // if (!Directory.fromUri(Uri.parse("${rootPath}/lib/generated")).existsSync()) {}
  Directory.fromUri(Uri.parse("${rootPath}/lib/generated")).createSync(recursive: true);

  //  代码生成 （不知道怎么获取插件包的地址，不能直接拷贝文件，改成了字符串输出到文件）
  // var supportSvgFile = File.fromUri(Uri.parse("${rootPath}/bin/svg_assets_support.dart"));
  // supportSvgFile.copySync("${rootPath}/lib/generated/svg_assets_support.dart"); // 拷贝svg支持
  File.fromUri(Uri.parse("${rootPath}/lib/generated/svg_assets_support.dart")).writeAsStringSync(svgSupportString);
  //  代码生成
  File.fromUri(Uri.parse("${rootPath}/lib/generated/r.assets.dart")).writeAsStringSync(resultCodeData);
  File.fromUri(Uri.parse("${rootPath}/lib/generated/r.text.dart")).writeAsStringSync(resultCodeData4Text);
  File.fromUri(Uri.parse("${rootPath}/lib/generated/r.string.dart")).writeAsStringSync(resultCodeData4String);
  File.fromUri(Uri.parse("${rootPath}/lib/generated/r.icon.dart")).writeAsStringSync(resultCodeData4Icon);

  // save yaml back;
  pubspec['auto_generate_desc'] = '有r插件自动生成的配置，更改pubspec会导致分析器重启，cache上一次的参数，减少重启频率';
  pubspec['auto_generate_assets'] = true;
  pubspec['auto_generate_reason'] = reason;
  pubspec['auto_generate_datetime'] = DateTime.now().millisecondsSinceEpoch;
  pubspec['auto_generate_readable'] = DateTime.now();
  File.fromUri(Uri.parse("${rootPath}/pubspec.yaml")).writeAsStringSync(json2yaml(
    pubspec,
    yamlStyle: YamlStyle.pubspecYaml,
  ));

  // // 更新pub库
  // Process.run('flutter', ['pub', 'get'], runInShell: true, workingDirectory: rootPath).then((ProcessResult pr) {
  //   mirrorLog.info('---' * 10);
  //   mirrorLog.info('exitCode:${pr.exitCode}');
  //   mirrorLog.info(pr.stdout);
  //   mirrorLog.info(pr.stderr);
  // });

  // // 格式化代码
  // Process.run('dartfmt', ['-w', 'lib/generated'], runInShell: true, workingDirectory: rootPath)
  //     .then((ProcessResult pr) {
  //   mirrorLog.info('---' * 10);
  //   mirrorLog.info('exitCode:${pr.exitCode}');
  //   mirrorLog.info(pr.stdout);
  //   mirrorLog.info(pr.stderr);
  // });
}
